/*
 * $RCSfile: DisplayFactory.java,v $
 *
 * Copyright (c) 1999-2002. Christian Heller. All Rights Reserved.
 *
 * This software is published under the GPL GNU General Public License.
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * http://www.resmedicinae.org
 * - Information in Medicine -
 */

package org.resmedicinae.resmedlib.component.view.swing;

import java.awt.BorderLayout;
import java.lang.*;
import org.resmedicinae.resmedlib.component.view.*;

/**
 * This class represents a display factory.
 *
 * @version $Revision: 1.8 $ $Date: 2002/08/19 09:11:18 $ $Author: chrissy $
 * @author Christian Heller <christian.heller@tuxtax.de>
 */
public class DisplayFactory extends Object {

    /** The window display mode. */
    public static final int WINDOW_DISPLAY = 0;

    /** The modal dialog display mode. */
    public static final int MODAL_DIALOG_DISPLAY = 1;

    /** The modeless dialog display mode. */
    public static final int MODELESS_DIALOG_DISPLAY = 2;

    /** The frame display mode. */
    public static final int FRAME_DISPLAY = 3;

    /** The internal frame display mode. */
    public static final int INTERNAL_FRAME_DISPLAY = 4;

    /** The tab page display mode. */
    public static final int TAB_PAGE_DISPLAY = 5;

    /**
     * Creates display.
     *
     * @param v the application view to create a display for
     * @param pv the parent application view
     * @return the root pane of the display
     */
    public ResRootPane createDisplay(SwingView v, SwingView pv) throws Exception {
        
        return null;
    }

    /**
     * Destroys display.
     *
     * @param v the application view whose display is to be destroyed
     * @param pv the parent application view
     * @param r the root pane of the application view
     * @exception NullPointerException if the root pane is null
     */
    public void destroyDisplay(SwingView v, SwingView pv, ResRootPane r) {
    }

    /**
     * Brings display to front.
     *
     * @param r the root pane of the application view whose display is to be brought to front
     */
    public void bringDisplayToFront(ResRootPane r) {
    }

    /**
     * Initializes the root pane.
     *
     * This method just takes values of the given application view
     * (menu bar, content pane) and adds them to the root pane.
     *
     * @param r the root pane
     * @param v the application view
     * @exception NullPointerException if the root pane or the swing view is null
     */
    protected void initializeRootPane(ResRootPane r, SwingView v) throws NullPointerException {

        if ((r != null) && (v != null)) {

            r.setJMenuBar(v.getMenuBar());
            r.getContentPane().setLayout(new BorderLayout());
            r.getContentPane().add((SwingView) v, BorderLayout.CENTER);

        } else {

            throw new NullPointerException("Could not initialize root pane. The root pane or the swing view is null.");
        }
    }

    /**
     * Finalizes the root pane.
     *
     * This method just takes values of the given root pane (menu bar, content pane)
     * and adds them to the application view.
     *
     * @param r the root pane
     * @param v the application view
     * @exception NullPointerException if the root pane or the swing view is null
     */
    protected void finalizeRootPane(ResRootPane r, SwingView v) throws NullPointerException {

        if ((r != null) && (v != null)) {

            v.setMenuBar((ResMenuBar) r.getJMenuBar());

        } else {

            throw new NullPointerException("Could not finalize root pane. The root pane or the swing view is null.");
        }
    }
}

